const base_url = "https://edt.telecomnancy.univ-lorraine.fr/static/";
//const base_url = "http://localhost:8000/static/";
current_selected = null;
is_loading = false;
groups = [
  'fise_1a',
  'fise_1a_g1',
  'fise_1a_g2',
  'fise_1a_g3',
  'fise_1a_g4',
  'fise_2a',
  'fise_2a_g1',
  'fise_2a_g2',
  'fise_2a_g3',
  'fise_2a_g4',
  'fise_2a_g5',
  'fise_2a_iamd',
  'fise_2a_il',
  'fise_2a_le',
  'fise_2a_sie',
  'fise_2a_iss',
  'fise_3a',
  'fise_3a_iamd',
  'fise_3a_il',
  'fise_3a_le',
  'fise_3a_sie',
  'fise_3a_iss',
  'fisa_1a',
  'fisa_2a',
  'fisa_3a'
];

function data_req (url) {
  return new Promise((resolve, reject) => {
    let req = null
    if (window.XDomainRequest) {
      req = new XDomainRequest();
    } else if (window.XMLHttpRequest) {
      req = new XMLHttpRequest();
    } else {
      alert("Votre navigateur ne gère pas l'AJAX cross-domain !");
      reject();
    }
    req.addEventListener('load', () => resolve(req.response))
    req.open('GET', url, true)
    req.send()
  })
}

function add_recur_events() {
    $('#calendar').fullCalendar('addEventSource', expand_recur_events)
}

async function load_ics(ics){
  try {
    const response = await data_req(ics.url)
    $('#calendar').fullCalendar('addEventSource', fc_events(response, ics.event_properties))
  } catch(e) {
    console.error(`Erreur de chargement du calendrier ${ics.url}`);
  }
}

function init_calendar() {
  $('#calendar').fullCalendar({
    defaultView: 'agendaWeek',
    businessHours: {
      start: '8:00',
      end: '18:00'
    },
    nowIndicator: true,
    locale: 'fr',
    timezone: 'Europe/Paris',
    minTime: '7:00',
    maxTime: '21:00',
    titleRangeSeparator: '-',
    weekNumbers: true,
    eventRender: (event, element) => {
      element.find('.fc-content').append('<div class="fc-event-room">' + event.room + '</div>');
      element.qtip({
          content: {
            text: '<div class="event-title">' + event.title + '</div>' +
                  '<div class="event-datetime"><span class="event-date">' + event.start.format('dddd D MMMM') + ' ⋅ </span><span class="event-timeslot">' + event.start.format('hh:mm') + ' à ' + event.end.format('hh:mm') + '</span></div>' +
                  (event.room?'<div class="event-location">&#8982; ' + event.room + '</div>':'') +
                  '<div class="event-description">' + event.description + '</div>'
          },
          style: {
            classes: 'qtip-bootstrap'
          },
          position: {
            my: 'top center',
            at: 'bottom center',
            target: element
          }
        });
    },
    slotLabelFormat:'HH:mm',
    columnHeaderHtml: function(d) {
      return '<span class="dayOfWeek">' + d.format('ddd') + '</span><span class="dayOfMonth">' + d.format('D') +'</span>';
    },
    hiddenDays: [ 0 ],
    allDaySlot: false,
    buttonText: {
      today: "aujourd'hui"
    },
    header: {
      left: '',
      center: '',
      right: 'title, prev, today, next'
    },
    footer: {
      left:   '', 
      center: '',
      right:  ''
    },
    height: 'auto'
  })
}

async function load_calendar(ressource) {
  await load_ics({url: base_url + ressource})
  add_recur_events()
}

function init_handlers() {
  for(group of groups){
    handler(group);
  }
}

function handler(ress) {
  $('#' + ress).click(async function() {
    if(is_loading) return;
    is_loading = true;
    $('#calendar').fullCalendar('removeEventSources');
    await load_calendar(ress + '.ics');
    if (current_selected != null) $(current_selected).removeClass('is-active');
    current_selected = $(this);
    $(this).addClass('is-active');
    history.replaceState({group:ress}, ress, `?group=${ress}`);
    is_loading = false;
  });
}

$(function() {
  init_handlers();
  init_calendar();
  
  let group = 'fise_1a';   // Default group
  let url = new URL(window.location.href);
  let params = url.searchParams ;

  if (params.has("group") && groups.includes(params.get("group")))
    group = params.get("group");

  $('#'+group).click();
});
